from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNDetectTextRectanglesRequest(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(
            Vision.VNDetectTextRectanglesRequest.reportCharacterBoxes
        )
        self.assertArgIsBOOL(
            Vision.VNDetectTextRectanglesRequest.setReportCharacterBoxes_, 0
        )

    def test_constants(self):
        self.assertEqual(Vision.VNDetectTextRectanglesRequestRevision1, 1)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVision(TestCase):
    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(Vision.VNVisionVersionNumber, float)



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import Vision  # noqa: F401
import objc


class TestVNFaceObservationAccepting(TestCase):
    @min_sdk_level("10.13")
    def testProtocols10_13(self):
        objc.protocolNamed("VNFaceObservationAccepting")



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNDetectBarcodesRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNDetectBarcodesRequestRevision1, 1)



import objc

from PyObjCTools.TestSupport import TestCase, min_sdk_level
import Vision


class TestVNRequestRevisionProviderHelper(Vision.NSObject):
    def requestRevision(self):
        return 1


class TestVNRequestRevisionProvider(TestCase):
    def test_methods(self):
        self.assertResultHasType(
            TestVNRequestRevisionProviderHelper.requestRevision, objc._C_NSUInteger
        )

    @min_sdk_level("10.14")
    def test_protocols(self):
        objc.protocolNamed("VNRequestRevisionProviding")



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNRequest(TestCase):
    def testConstants(self):
        self.assertEqual(Vision.VNRequestTrackingLevelAccurate, 0)
        self.assertEqual(Vision.VNRequestTrackingLevelFast, 1)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(Vision.VNTrackingRequest.isLastFrame)
        self.assertArgIsBOOL(Vision.VNTrackingRequest.setLastFrame_, 0)
        self.assertArgIsBlock(
            Vision.VNTrackingRequest.initWithCompletionHandler_, 0, b"v@@"
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNGenerateOpticalFlowRequest(TestCase):
    def testConstants(self):
        self.assertEqual(Vision.VNGenerateOpticalFlowRequestComputationAccuracyLow, 0)
        self.assertEqual(
            Vision.VNGenerateOpticalFlowRequestComputationAccuracyMedium, 1
        )
        self.assertEqual(Vision.VNGenerateOpticalFlowRequestComputationAccuracyHigh, 2)
        self.assertEqual(
            Vision.VNGenerateOpticalFlowRequestComputationAccuracyVeryHigh, 3
        )

    @min_os_level("10.16")
    def testConstants10_16(self):
        self.assertEqual(Vision.VNGenerateOpticalFlowRequestRevision1, 1)



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNError(TestCase):
    def testConstants(self):
        self.assertEqual(Vision.VNErrorOK, 0)
        self.assertEqual(Vision.VNErrorRequestCancelled, 1)
        self.assertEqual(Vision.VNErrorInvalidFormat, 2)
        self.assertEqual(Vision.VNErrorOperationFailed, 3)
        self.assertEqual(Vision.VNErrorOutOfBoundsError, 4)
        self.assertEqual(Vision.VNErrorInvalidOption, 5)
        self.assertEqual(Vision.VNErrorIOError, 6)
        self.assertEqual(Vision.VNErrorMissingOption, 7)
        self.assertEqual(Vision.VNErrorNotImplemented, 8)
        self.assertEqual(Vision.VNErrorInternalError, 9)
        self.assertEqual(Vision.VNErrorOutOfMemory, 10)
        self.assertEqual(Vision.VNErrorUnknownError, 11)
        self.assertEqual(Vision.VNErrorInvalidOperation, 12)
        self.assertEqual(Vision.VNErrorInvalidImage, 13)
        self.assertEqual(Vision.VNErrorInvalidArgument, 14)
        self.assertEqual(Vision.VNErrorInvalidModel, 15)
        self.assertEqual(Vision.VNErrorUnsupportedRevision, 16)
        self.assertEqual(Vision.VNErrorDataUnavailable, 17)
        self.assertEqual(Vision.VNErrorTimeStampNotFound, 18)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNCoreMLRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNCoreMLRequestRevision1, 1)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsOut(Vision.VNCoreMLModel.modelForMLModel_error_, 1)

        self.assertArgIsBlock(
            Vision.VNCoreMLRequest.initWithModel_completionHandler_, 1, b"v@@"
        )
        self.assertArgIsBlock(
            Vision.VNCoreMLRequest.initWithCompletionHandler_, 0, b"v@@"
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNRecognizeAnimalsRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNRecognizeAnimalsRequestRevision1, 1)

    @min_os_level("10.15")
    def test_constants10_15(self):
        self.assertIsInstance(Vision.VNAnimalIdentifierDog, str)
        self.assertIsInstance(Vision.VNAnimalIdentifierCat, str)

    @min_os_level("10.15")
    def test_methods10_15(self):
        self.assertArgIsOut(
            Vision.VNRecognizeAnimalsRequest.knownAnimalIdentifiersForRevision_error_, 1
        )



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNDetectFaceCaptureQualityRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNDetectFaceCaptureQualityRequestRevision1, 1)
        self.assertEqual(Vision.VNDetectFaceCaptureQualityRequestRevision2, 2)
        self.assertEqual(Vision.VNDetectFaceQualityRequestRevision1, 1)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNRequestHandler(TestCase):
    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(Vision.VNImageOptionProperties, str)
        self.assertIsInstance(Vision.VNImageOptionCameraIntrinsics, str)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(Vision.VNImageRequestHandler.performRequests_error_)
        self.assertArgIsOut(Vision.VNImageRequestHandler.performRequests_error_, 1)

        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onCVPixelBuffer_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onCVPixelBuffer_error_, 2
        )
        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onCVPixelBuffer_orientation_error_  # noqa: B950
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onCVPixelBuffer_orientation_error_,  # noqa: B950
            3,
        )

        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onCGImage_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onCGImage_error_, 2
        )
        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onCGImage_orientation_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onCGImage_orientation_error_,
            3,
        )

        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onCIImage_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onCIImage_error_, 2
        )
        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onCIImage_orientation_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onCIImage_orientation_error_,
            3,
        )

        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onImageURL_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onImageURL_error_, 2
        )
        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onImageURL_orientation_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onImageURL_orientation_error_,
            3,
        )

        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onImageData_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onImageData_error_, 2
        )
        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onImageData_orientation_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onImageData_orientation_error_,
            3,
        )

    @min_os_level("10.16")
    def testMethods10_16(self):
        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onCMSampleBuffer_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onCMSampleBuffer_error_, 2
        )

        self.assertResultIsBOOL(
            Vision.VNSequenceRequestHandler.performRequests_onCMSampleBuffer_orientation_error_
        )
        self.assertArgIsOut(
            Vision.VNSequenceRequestHandler.performRequests_onCMSampleBuffer_orientation_error_,
            3,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNRecognizeTextRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNRequestTextRecognitionLevelAccurate, 0)
        self.assertEqual(Vision.VNRequestTextRecognitionLevelFast, 1)

        self.assertEqual(Vision.VNRecognizeTextRequestRevision1, 1)
        self.assertEqual(Vision.VNRecognizeTextRequestRevision2, 2)

    @min_os_level("10.15")
    def test_methods(self):
        self.assertArgIsOut(
            Vision.VNRecognizeTextRequest.supportedRecognitionLanguagesForTextRecognitionLevel_revision_error_,  # noqa: B950
            2,
        )

        self.assertResultIsBOOL(Vision.VNRecognizeTextRequest.usesLanguageCorrection)
        self.assertArgIsBOOL(
            Vision.VNRecognizeTextRequest.setUsesLanguageCorrection_, 0
        )



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNDetectRectanglesRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNDetectRectanglesRequestRevision1, 1)



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNGenerateObjectnessBasedSaliencyImageRequest(TestCase):
    def test_constants(self):
        self.assertEqual(
            Vision.VNGenerateObjectnessBasedSaliencyImageRequestRevision1, 1
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNDetectContoursRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNDetectContourRequestRevision1, 1)

    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertResultIsBOOL(Vision.VNDetectContoursRequest.detectDarkOnLight)
        self.assertArgIsBOOL(Vision.VNDetectContoursRequest.setDetectDarkOnLight_, 0)

        self.assertResultIsBOOL(Vision.VNDetectContoursRequest.detectsDarkOnLight)
        self.assertArgIsBOOL(Vision.VNDetectContoursRequest.setDetectsDarkOnLight_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNObservation(TestCase):
    @min_os_level("10.16")
    def test_constants10_16(self):
        self.assertIsInstance(Vision.VNRecognizedPointGroupKeyAll, str)

    @min_os_level("10.15")
    def test_methods(self):
        self.assertResultIsBOOL(
            Vision.VNClassificationObservation.hasMinimumRecall_forPrecision_
        )
        self.assertResultIsBOOL(
            Vision.VNClassificationObservation.hasMinimumPrecision_forRecall_
        )

        self.assertArgIsOut(Vision.VNRecognizedText.boundingBoxForRange_error_, 1)

        self.assertResultIsBOOL(
            Vision.VNFeaturePrintObservation.computeDistance_toFeaturePrintObservation_error_  # noqa: B950
        )
        self.assertArgIsOut(
            Vision.VNFeaturePrintObservation.computeDistance_toFeaturePrintObservation_error_,  # noqa: B950
            2,
        )

    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertArgIsOut(Vision.VNContoursObservation.contourAtIndex_error_, 1)
        self.assertArgIsOut(Vision.VNContoursObservation.contourAtIndexPath_error_, 1)

        self.assertArgIsOut(
            Vision.VNRecognizedPointsObservation.recognizedPointsForGroupKey_error_, 1
        )
        self.assertArgIsOut(
            Vision.VNRecognizedPointsObservation.keypointsMultiArrayAndReturnError_, 0
        )



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNGenerateAttentionBasedSaliencyImageRequest(TestCase):
    def test_constants(self):
        self.assertEqual(
            Vision.VNGenerateAttentionBasedSaliencyImageRequestRevision1, 1
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNTypes(TestCase):
    def testConstants(self):
        self.assertEqual(Vision.VNImageCropAndScaleOptionCenterCrop, 0)
        self.assertEqual(Vision.VNImageCropAndScaleOptionScaleFit, 1)
        self.assertEqual(Vision.VNImageCropAndScaleOptionScaleFill, 2)

        self.assertEqual(Vision.VNElementTypeUnknown, 0)
        self.assertEqual(Vision.VNElementTypeFloat, 1)
        self.assertEqual(Vision.VNElementTypeDouble, 2)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(Vision.VNBarcodeSymbologyAztec, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyAztec, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyCode39Checksum, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyCode39FullASCII, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyCode39FullASCIIChecksum, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyCode93, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyCode93i, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyCode128, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyDataMatrix, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyEAN8, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyEAN13, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyI2of5, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyI2of5Checksum, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyITF14, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyPDF417, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyQR, str)
        self.assertIsInstance(Vision.VNBarcodeSymbologyUPCE, str)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNTrackRectangleRequest(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(
            Vision.VNTrackRectangleRequest.initWithRectangleObservation_completionHandler_,
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTrackRectangleRequest.initWithCompletionHandler_, 0, b"v@@"
        )

    def test_constants(self):
        self.assertEqual(Vision.VNTrackRectangleRequestRevision1, 1)



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNDetectFaceRectanglesRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNDetectFaceRectanglesRequestRevision1, 1)
        self.assertEqual(Vision.VNDetectFaceRectanglesRequestRevision2, 2)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNTargetedImageRequest(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCVPixelBuffer_completionHandler_,
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCGImage_completionHandler_,
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCIImage_completionHandler_,
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCIImage_options_completionHandler_,  # noqa: B950
            2,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCIImage_orientation_options_completionHandler_,  # noqa: B950
            3,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedImageURL_completionHandler_,
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedImageURL_options_completionHandler_,  # noqa: B950
            2,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedImageURL_orientation_options_completionHandler_,  # noqa: B950
            3,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedImageData_completionHandler_,
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedImageData_options_completionHandler_,  # noqa: B950
            2,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedImageData_orientation_options_completionHandler_,  # noqa: B950
            3,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithCompletionHandler_, 0, b"v@@"
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCVPixelBuffer_options_completionHandler_,  # noqa: B950
            2,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCVPixelBuffer_orientation_options_completionHandler_,  # noqa: B950
            3,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCGImage_options_completionHandler_,  # noqa: B950
            2,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCGImage_orientation_options_completionHandler_,  # noqa: B950
            3,
            b"v@@",
        )

    @min_os_level("10.16")
    def testMethods10_16(self):
        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCMSampleBuffer_options_completionHandler_,
            2,
            b"v@@",
        )

        self.assertArgIsBlock(
            Vision.VNTargetedImageRequest.initWithTargetedCMSampleBuffer_orientation_options_completionHandler_,
            3,
            b"v@@",
        )



import objc
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import Vision

VNRequestProgressHandler = b"v@d@"


class TestVNRequestHelper(Vision.NSObject):
    def progressHandler(self):
        return 1

    def setProgressHandler_(self, v):
        pass

    def indeterminate(self):
        return 1


class TestVNRequest(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(Vision.VNRequest.initWithCompletionHandler_, 0, b"v@@")
        self.assertResultIsBOOL(Vision.VNRequest.preferBackgroundProcessing)
        self.assertArgIsBOOL(Vision.VNRequest.setPreferBackgroundProcessing_, 0)
        self.assertResultIsBOOL(Vision.VNRequest.usesCPUOnly)
        self.assertArgIsBOOL(Vision.VNRequest.setUsesCPUOnly_, 0)
        self.assertResultIsBlock(Vision.VNRequest.completionHandler, b"v@@")

        self.assertResultIsBlock(
            TestVNRequestHelper.progressHandler, VNRequestProgressHandler
        )
        self.assertArgIsBlock(
            TestVNRequestHelper.setProgressHandler_, 0, VNRequestProgressHandler
        )
        self.assertResultIsBOOL(TestVNRequestHelper.indeterminate)

    def test_constants(self):
        self.assertEqual(Vision.VNRequestRevisionUnspecified, 0)

    @min_sdk_level("10.15")
    def testProtocols(self):
        objc.protocolNamed("VNRequestProgressProviding")



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNFaceLandmarks(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsVariableSize(Vision.VNFaceLandmarkRegion2D.normalizedPoints)
        self.assertResultIsVariableSize(
            Vision.VNFaceLandmarkRegion2D.pointsInImageOfSize_
        )



"""
Python mapping for the Vision framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""

import sys

import CoreML
import Foundation
import objc
import Quartz
from Vision import _metadata
from Vision import _Vision

sys.modules["Vision"] = mod = objc.ObjCLazyModule(
    "Vision",
    "com.apple.Vision",
    objc.pathForFramework("/System/Library/Frameworks/Vision.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "objc": objc,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (_Vision, Foundation, Quartz, CoreML),
)


del sys.modules["Vision._metadata"]



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision
import Quartz
import objc


class TestVNUtils(TestCase):
    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(Vision.VNNormalizedIdentityRect, Quartz.CGRect)

    @min_os_level("10.13")
    def testFunctions(self):
        self.assertResultHasType(Vision.VNNormalizedRectIsIdentityRect, objc._C_BOOL)
        self.assertArgHasType(
            Vision.VNNormalizedRectIsIdentityRect, 0, Quartz.CGRect.__typestr__
        )

        self.assertResultHasType(
            Vision.VNImagePointForNormalizedPoint, Quartz.CGPoint.__typestr__
        )
        self.assertArgHasType(
            Vision.VNImagePointForNormalizedPoint, 0, Quartz.CGPoint.__typestr__
        )
        self.assertArgHasType(Vision.VNImagePointForNormalizedPoint, 1, objc._C_ULNG)
        self.assertArgHasType(Vision.VNImagePointForNormalizedPoint, 2, objc._C_ULNG)

        self.assertResultHasType(
            Vision.VNImageRectForNormalizedRect, Quartz.CGRect.__typestr__
        )
        self.assertArgHasType(
            Vision.VNImageRectForNormalizedRect, 0, Quartz.CGRect.__typestr__
        )
        self.assertArgHasType(Vision.VNImageRectForNormalizedRect, 1, objc._C_ULNG)
        self.assertArgHasType(Vision.VNImageRectForNormalizedRect, 2, objc._C_ULNG)

        self.assertResultHasType(
            Vision.VNNormalizedRectForImageRect, Quartz.CGRect.__typestr__
        )
        self.assertArgHasType(
            Vision.VNNormalizedRectForImageRect, 0, Quartz.CGRect.__typestr__
        )
        self.assertArgHasType(Vision.VNNormalizedRectForImageRect, 1, objc._C_ULNG)
        self.assertArgHasType(Vision.VNNormalizedRectForImageRect, 2, objc._C_ULNG)

        # Vector types
        self.assertFalse(
            hasattr(Vision, "VNNormalizedFaceBoundingBoxPointForLandmarkPoint")
        )
        # self.assertResultHasType(Vision.VNNormalizedFaceBoundingBoxPointForLandmarkPoint, Quartz.CGPoint.__typestr__)  # noqa: B950
        # self.assertArgHasType(Vision.VNNormalizedRectForImageRect, 0, vector_float2)
        # self.assertArgHasType(Vision.VNNormalizedRectForImageRect, 1, Quartz.CGRect.__typestr__)  # noqa: B950
        # self.assertArgHasType(Vision.VNNormalizedRectForImageRect, 2, objc._C_ULNG)
        # self.assertArgHasType(Vision.VNNormalizedRectForImageRect, 3, objc._C_ULNG)

        # Vector types
        self.assertFalse(hasattr(Vision, "VNImagePointForFaceLandmarkPoint"))
        # self.assertResultHasType(Vision.VNImagePointForFaceLandmarkPoint, Quartz.CGPoint.__typestr__)  # noqa: B950
        # self.assertArgHasType(Vision.VNImagePointForFaceLandmarkPoint, 0, vector_float2)
        # self.assertArgHasType(Vision.VNImagePointForFaceLandmarkPoint, 1, Quartz.CGRect.__typestr__)  # noqa: B950
        # self.assertArgHasType(Vision.VNImagePointForFaceLandmarkPoint, 2, objc._C_ULNG)
        # self.assertArgHasType(Vision.VNImagePointForFaceLandmarkPoint, 3, objc._C_ULNG)

    @min_os_level("10.15")
    def testFunctions10_15(self):
        Vision.VNElementTypeSize

    @min_os_level("10.16")
    def testFunctions10_16(self):
        Vision.VNNormalizedPointForImagePoint



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNImageRegistrationRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNTranslationalImageRegistrationRequestRevision1, 1)
        self.assertEqual(Vision.VNHomographicImageRegistrationRequestRevision1, 1)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNDetectTrajectoriesRequest(TestCase):
    @min_os_level("10.16")
    def test_constants10_16(self):
        self.assertEqual(Vision.VNDetectTrajectoriesRequestRevision1, 1)

    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertArgIsBlock(
            Vision.VNDetectTrajectoriesRequest.initWithFrameAnalysisSpacing_completionHandler_,
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNDetectTrajectoriesRequest.initWithFrameAnalysisSpacing_trajectoryLength_completionHandler_,
            2,
            b"v@@",
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNTrackObjectRequest(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(
            Vision.VNTrackObjectRequest.initWithDetectedObjectObservation_completionHandler_,  # noqa: B950
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            Vision.VNTrackObjectRequest.initWithCompletionHandler_, 0, b"v@@"
        )

    def test_constants(self):
        self.assertEqual(Vision.VNTrackObjectRequestRevision1, 1)
        self.assertEqual(Vision.VNTrackObjectRequestRevision2, 2)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNVideoProcessor(TestCase):
    @min_os_level("10.16")
    def test_constants(self):
        self.assertIsInstance(Vision.VNVideoProcessingOptionFrameCadence, str)
        self.assertIsInstance(Vision.VNVideoProcessingOptionTimeInterval, str)

    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertResultIsBOOL(
            Vision.VNVideoProcessor.addRequest_withProcessingOptions_error_
        )
        self.assertArgIsOut(
            Vision.VNVideoProcessor.addRequest_withProcessingOptions_error_, 2
        )
        self.assertResultIsBOOL(
            Vision.VNVideoProcessor.addRequest_processingOptions_error_
        )
        self.assertArgIsOut(
            Vision.VNVideoProcessor.addRequest_processingOptions_error_, 2
        )

        self.assertResultIsBOOL(Vision.VNVideoProcessor.removeRequest_error_)
        self.assertArgIsOut(Vision.VNVideoProcessor.removeRequest_error_, 1)

        self.assertResultIsBOOL(Vision.VNVideoProcessor.analyzeWithTimeRange_error_)
        self.assertArgIsOut(Vision.VNVideoProcessor.analyzeWithTimeRange_error_, 1)

        self.assertResultIsBOOL(Vision.VNVideoProcessor.analyzeTimeRange_error_)
        self.assertArgIsOut(Vision.VNVideoProcessor.analyzeTimeRange_error_, 1)



# This file is generated by objective.metadata
#
# Last update: Thu Nov 12 21:49:11 2020
#
# flake8: noqa

import objc, sys

if sys.maxsize > 2 ** 32:
    def sel32or64(a, b): return b
else:
    def sel32or64(a, b): return a

misc = {
}
constants = '''$VNAnimalDetectorCat$VNAnimalDetectorDog$VNAnimalIdentifierCat$VNAnimalIdentifierDog$VNBarcodeSymbologyAztec$VNBarcodeSymbologyCode128$VNBarcodeSymbologyCode39$VNBarcodeSymbologyCode39Checksum$VNBarcodeSymbologyCode39FullASCII$VNBarcodeSymbologyCode39FullASCIIChecksum$VNBarcodeSymbologyCode93$VNBarcodeSymbologyCode93i$VNBarcodeSymbologyDataMatrix$VNBarcodeSymbologyEAN13$VNBarcodeSymbologyEAN8$VNBarcodeSymbologyI2of5$VNBarcodeSymbologyI2of5Checksum$VNBarcodeSymbologyITF14$VNBarcodeSymbologyPDF417$VNBarcodeSymbologyQR$VNBarcodeSymbologyUPCE$VNBodyLandmarkKeyLeftAnkle$VNBodyLandmarkKeyLeftEar$VNBodyLandmarkKeyLeftElbow$VNBodyLandmarkKeyLeftEye$VNBodyLandmarkKeyLeftHip$VNBodyLandmarkKeyLeftKnee$VNBodyLandmarkKeyLeftShoulder$VNBodyLandmarkKeyLeftWrist$VNBodyLandmarkKeyNeck$VNBodyLandmarkKeyNose$VNBodyLandmarkKeyRightAnkle$VNBodyLandmarkKeyRightEar$VNBodyLandmarkKeyRightElbow$VNBodyLandmarkKeyRightEye$VNBodyLandmarkKeyRightHip$VNBodyLandmarkKeyRightKnee$VNBodyLandmarkKeyRightShoulder$VNBodyLandmarkKeyRightWrist$VNBodyLandmarkKeyRoot$VNBodyLandmarkRegionKeyFace$VNBodyLandmarkRegionKeyLeftArm$VNBodyLandmarkRegionKeyLeftLeg$VNBodyLandmarkRegionKeyRightArm$VNBodyLandmarkRegionKeyRightLeg$VNBodyLandmarkRegionKeyTorso$VNErrorDomain$VNHandLandmarkKeyIndexDIP$VNHandLandmarkKeyIndexMCP$VNHandLandmarkKeyIndexPIP$VNHandLandmarkKeyIndexTIP$VNHandLandmarkKeyLittleDIP$VNHandLandmarkKeyLittleMCP$VNHandLandmarkKeyLittlePIP$VNHandLandmarkKeyLittleTIP$VNHandLandmarkKeyMiddleDIP$VNHandLandmarkKeyMiddleMCP$VNHandLandmarkKeyMiddlePIP$VNHandLandmarkKeyMiddleTIP$VNHandLandmarkKeyRingDIP$VNHandLandmarkKeyRingMCP$VNHandLandmarkKeyRingPIP$VNHandLandmarkKeyRingTIP$VNHandLandmarkKeyThumbCMC$VNHandLandmarkKeyThumbIP$VNHandLandmarkKeyThumbMP$VNHandLandmarkKeyThumbTIP$VNHandLandmarkKeyWrist$VNHandLandmarkRegionKeyIndexFinger$VNHandLandmarkRegionKeyLittleFinger$VNHandLandmarkRegionKeyMiddleFinger$VNHandLandmarkRegionKeyRingFinger$VNHandLandmarkRegionKeyThumb$VNHumanBodyPoseObservationJointNameLeftAnkle$VNHumanBodyPoseObservationJointNameLeftEar$VNHumanBodyPoseObservationJointNameLeftElbow$VNHumanBodyPoseObservationJointNameLeftEye$VNHumanBodyPoseObservationJointNameLeftHip$VNHumanBodyPoseObservationJointNameLeftKnee$VNHumanBodyPoseObservationJointNameLeftShoulder$VNHumanBodyPoseObservationJointNameLeftWrist$VNHumanBodyPoseObservationJointNameNeck$VNHumanBodyPoseObservationJointNameNose$VNHumanBodyPoseObservationJointNameRightAnkle$VNHumanBodyPoseObservationJointNameRightEar$VNHumanBodyPoseObservationJointNameRightElbow$VNHumanBodyPoseObservationJointNameRightEye$VNHumanBodyPoseObservationJointNameRightHip$VNHumanBodyPoseObservationJointNameRightKnee$VNHumanBodyPoseObservationJointNameRightShoulder$VNHumanBodyPoseObservationJointNameRightWrist$VNHumanBodyPoseObservationJointNameRoot$VNHumanBodyPoseObservationJointsGroupNameAll$VNHumanBodyPoseObservationJointsGroupNameFace$VNHumanBodyPoseObservationJointsGroupNameLeftArm$VNHumanBodyPoseObservationJointsGroupNameLeftLeg$VNHumanBodyPoseObservationJointsGroupNameRightArm$VNHumanBodyPoseObservationJointsGroupNameRightLeg$VNHumanBodyPoseObservationJointsGroupNameTorso$VNHumanHandPoseObservationJointNameIndexDIP$VNHumanHandPoseObservationJointNameIndexMCP$VNHumanHandPoseObservationJointNameIndexPIP$VNHumanHandPoseObservationJointNameIndexTip$VNHumanHandPoseObservationJointNameLittleDIP$VNHumanHandPoseObservationJointNameLittleMCP$VNHumanHandPoseObservationJointNameLittlePIP$VNHumanHandPoseObservationJointNameLittleTip$VNHumanHandPoseObservationJointNameMiddleDIP$VNHumanHandPoseObservationJointNameMiddleMCP$VNHumanHandPoseObservationJointNameMiddlePIP$VNHumanHandPoseObservationJointNameMiddleTip$VNHumanHandPoseObservationJointNameRingDIP$VNHumanHandPoseObservationJointNameRingMCP$VNHumanHandPoseObservationJointNameRingPIP$VNHumanHandPoseObservationJointNameRingTip$VNHumanHandPoseObservationJointNameThumbCMC$VNHumanHandPoseObservationJointNameThumbIP$VNHumanHandPoseObservationJointNameThumbMP$VNHumanHandPoseObservationJointNameThumbTip$VNHumanHandPoseObservationJointNameWrist$VNHumanHandPoseObservationJointsGroupNameAll$VNHumanHandPoseObservationJointsGroupNameIndexFinger$VNHumanHandPoseObservationJointsGroupNameLittleFinger$VNHumanHandPoseObservationJointsGroupNameMiddleFinger$VNHumanHandPoseObservationJointsGroupNameRingFinger$VNHumanHandPoseObservationJointsGroupNameThumb$VNImageOptionCIContext$VNImageOptionCameraIntrinsics$VNImageOptionProperties$VNNormalizedIdentityRect@{CGRect={CGPoint=dd}{CGSize=dd}}$VNRecognizedPointGroupKeyAll$VNVideoProcessingOptionFrameCadence$VNVideoProcessingOptionTimeInterval$VNVisionVersionNumber@d$'''
enums = '''$VNClassifyImageRequestRevision1@1$VNCoreMLRequestRevision1@1$VNDetectAnimalRectanglesRequestRevision1@1$VNDetectBarcodesRequestRevision1@1$VNDetectContourRequestRevision1@1$VNDetectFaceCaptureQualityRequestRevision1@1$VNDetectFaceCaptureQualityRequestRevision2@2$VNDetectFaceLandmarksRequestRevision1@1$VNDetectFaceLandmarksRequestRevision2@2$VNDetectFaceLandmarksRequestRevision3@3$VNDetectFaceQualityRequestRevision1@1$VNDetectFaceRectanglesRequestRevision1@1$VNDetectFaceRectanglesRequestRevision2@2$VNDetectHorizonRequestRevision1@1$VNDetectHumanBodyPoseRequestRevision1@1$VNDetectHumanHandPoseRequestRevision1@1$VNDetectHumanRectanglesRequestRevision1@1$VNDetectRectanglesRequestRevision1@1$VNDetectTextRectanglesRequestRevision1@1$VNDetectTrajectoriesRequestRevision1@1$VNElementTypeDouble@2$VNElementTypeFloat@1$VNElementTypeUnknown@0$VNErrorDataUnavailable@17$VNErrorIOError@6$VNErrorInternalError@9$VNErrorInvalidArgument@14$VNErrorInvalidFormat@2$VNErrorInvalidImage@13$VNErrorInvalidModel@15$VNErrorInvalidOperation@12$VNErrorInvalidOption@5$VNErrorMissingOption@7$VNErrorNotImplemented@8$VNErrorOK@0$VNErrorOperationFailed@3$VNErrorOutOfBoundsError@4$VNErrorOutOfMemory@10$VNErrorRequestCancelled@1$VNErrorTimeStampNotFound@18$VNErrorUnknownError@11$VNErrorUnsupportedRevision@16$VNGenerateAttentionBasedSaliencyImageRequestRevision1@1$VNGenerateImageFeaturePrintRequestRevision1@1$VNGenerateObjectnessBasedSaliencyImageRequestRevision1@1$VNGenerateOpticalFlowRequestComputationAccuracyHigh@2$VNGenerateOpticalFlowRequestComputationAccuracyLow@0$VNGenerateOpticalFlowRequestComputationAccuracyMedium@1$VNGenerateOpticalFlowRequestComputationAccuracyVeryHigh@3$VNGenerateOpticalFlowRequestRevision1@1$VNHomographicImageRegistrationRequestRevision1@1$VNImageCropAndScaleOptionCenterCrop@0$VNImageCropAndScaleOptionScaleFill@2$VNImageCropAndScaleOptionScaleFit@1$VNRecognizeAnimalsRequestRevision1@1$VNRecognizeTextRequestRevision1@1$VNRecognizeTextRequestRevision2@2$VNRequestFaceLandmarksConstellation65Points@1$VNRequestFaceLandmarksConstellation76Points@2$VNRequestFaceLandmarksConstellationNotDefined@0$VNRequestRevisionUnspecified@0$VNRequestTextRecognitionLevelAccurate@0$VNRequestTextRecognitionLevelFast@1$VNRequestTrackingLevelAccurate@0$VNRequestTrackingLevelFast@1$VNTrackObjectRequestRevision1@1$VNTrackObjectRequestRevision2@2$VNTrackRectangleRequestRevision1@1$VNTranslationalImageRegistrationRequestRevision1@1$'''
misc.update({})
functions={'VNImageRectForNormalizedRect': (b'{CGRect={CGPoint=dd}{CGSize=dd}}{CGRect={CGPoint=dd}{CGSize=dd}}QQ',), 'VNNormalizedPointForImagePoint': (b'{CGPoint=dd}{CGPoint=dd}QQ',), 'VNNormalizedRectIsIdentityRect': (b'B{CGRect={CGPoint=dd}{CGSize=dd}}',), 'VNNormalizedRectForImageRect': (b'{CGRect={CGPoint=dd}{CGSize=dd}}{CGRect={CGPoint=dd}{CGSize=dd}}QQ',), 'VNImagePointForNormalizedPoint': (b'{CGPoint=dd}{CGPoint=dd}QQ',), 'VNElementTypeSize': (b'QQ',)}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(b'NSObject', b'indeterminate', {'required': True, 'retval': {'type': 'Z'}})
    r(b'NSObject', b'inputFaceObservations', {'required': True, 'retval': {'type': b'@'}})
    r(b'NSObject', b'progressHandler', {'required': True, 'retval': {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'd'}, 3: {'type': b'@'}}}, 'type': '@?'}})
    r(b'NSObject', b'requestRevision', {'required': True, 'retval': {'type': 'Q'}})
    r(b'NSObject', b'setInputFaceObservations:', {'required': True, 'retval': {'type': b'v'}, 'arguments': {2: {'type': b'@'}}})
    r(b'NSObject', b'setProgressHandler:', {'required': True, 'retval': {'type': b'v'}, 'arguments': {2: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'd'}, 3: {'type': b'@'}}}, 'type': '@?'}}})
    r(b'VNCircle', b'containsPoint:', {'retval': {'type': b'Z'}})
    r(b'VNCircle', b'containsPoint:inCircumferentialRingOfWidth:', {'retval': {'type': b'Z'}})
    r(b'VNClassificationObservation', b'hasMinimumPrecision:forRecall:', {'retval': {'type': 'Z'}})
    r(b'VNClassificationObservation', b'hasMinimumRecall:forPrecision:', {'retval': {'type': 'Z'}})
    r(b'VNClassifyImageRequest', b'knownClassificationsForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNContour', b'childContourAtIndex:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNContour', b'polygonApproximationWithEpsilon:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNContoursObservation', b'contourAtIndex:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNContoursObservation', b'contourAtIndexPath:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNCoreMLModel', b'modelForMLModel:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNCoreMLRequest', b'initWithCompletionHandler:', {'arguments': {2: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNCoreMLRequest', b'initWithModel:completionHandler:', {'arguments': {3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNDetectContoursRequest', b'detectDarkOnLight', {'retval': {'type': b'Z'}})
    r(b'VNDetectContoursRequest', b'detectsDarkOnLight', {'retval': {'type': b'Z'}})
    r(b'VNDetectContoursRequest', b'setDetectDarkOnLight:', {'arguments': {2: {'type': b'Z'}}})
    r(b'VNDetectContoursRequest', b'setDetectsDarkOnLight:', {'arguments': {2: {'type': b'Z'}}})
    r(b'VNDetectFaceLandmarksRequest', b'revision:supportsConstellation:', {'retval': {'type': b'Z'}})
    r(b'VNDetectHumanBodyPoseRequest', b'supportedIdentifiedPointGroupKeysForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanBodyPoseRequest', b'supportedIdentifiedPointKeysForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanBodyPoseRequest', b'supportedJointNamesForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanBodyPoseRequest', b'supportedJointsGroupNamesForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanBodyPoseRequest', b'supportedRecognizedPointGroupKeysForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanBodyPoseRequest', b'supportedRecognizedPointKeysForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanHandPoseRequest', b'supportedIdentifiedPointGroupKeysForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanHandPoseRequest', b'supportedIdentifiedPointKeysForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanHandPoseRequest', b'supportedJointNamesForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanHandPoseRequest', b'supportedJointsGroupNamesForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanHandPoseRequest', b'supportedRecognizedPointGroupKeysForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectHumanHandPoseRequest', b'supportedRecognizedPointKeysForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNDetectTextRectanglesRequest', b'reportCharacterBoxes', {'retval': {'type': 'Z'}})
    r(b'VNDetectTextRectanglesRequest', b'setReportCharacterBoxes:', {'arguments': {2: {'type': 'Z'}}})
    r(b'VNDetectTrajectoriesRequest', b'initWithFrameAnalysisSpacing:completionHandler:', {'arguments': {2: {'type': b'{_CMTime=qiIq}'}, 3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNDetectTrajectoriesRequest', b'initWithFrameAnalysisSpacing:trajectoryLength:completionHandler:', {'arguments': {2: {'type': b'{_CMTime=qiIq}'}, 4: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNFaceLandmarkRegion2D', b'normalizedPoints', {'retval': {'c_array_of_variable_length': True}})
    r(b'VNFaceLandmarkRegion2D', b'pointsInImageOfSize:', {'retval': {'c_array_of_variable_length': True}})
    r(b'VNFeaturePrintObservation', b'computeDistance:toFeaturePrintObservation:error:', {'retval': {'type': 'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNGeometryUtils', b'boundingCircleForContour:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNGeometryUtils', b'boundingCircleForPoints:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNGeometryUtils', b'boundingCircleForSIMDPoints:pointCount:error:', {'arguments': {2: {'type_modifier': b'n', 'c_array_length_in_arg': 3}, 4: {'type_modifier': b'o'}}})
    r(b'VNGeometryUtils', b'calculateArea:forContour:orientedArea:error:', {'retval': {'type': b'Z'}, 'arguments': {2: {'type_modifier': b'n'}, 4: {'type': b'Z'}, 5: {'type_modifier': b'o'}}})
    r(b'VNGeometryUtils', b'calculatePerimeter:forContour:error:', {'retval': {'type': b'Z'}, 'arguments': {2: {'type_modifier': b'n'}, 4: {'type_modifier': b'o'}}})
    r(b'VNHumanBodyPoseObservation', b'recognizedPointForJointName:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNHumanBodyPoseObservation', b'recognizedPointsForJointsGroupName:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNHumanHandPoseObservation', b'recognizedPointForJointName:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNHumanHandPoseObservation', b'recognizedPointsForJointsGroupName:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNImageHomographicAlignmentObservation', b'setWarpTransform:', {'arguments': {2: {'type': b'{_matrix_float3x3=?}'}}})
    r(b'VNImageHomographicAlignmentObservation', b'warpTransform', {'retval': {'type': b'{_matrix_float3x3=?}'}})
    r(b'VNImageRequestHandler', b'performRequests:error:', {'retval': {'type': 'Z'}, 'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNObservation', b'timeRange', {'retval': {'type': b'{_CMTimeRange={_CMTime=qiIq}{_CMTime=qiIq}}'}})
    r(b'VNRecognizeAnimalsRequest', b'knownAnimalIdentifiersForRevision:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNRecognizeTextRequest', b'setUsesLanguageCorrection:', {'arguments': {2: {'type': 'Z'}}})
    r(b'VNRecognizeTextRequest', b'supportedRecognitionLanguagesForTextRecognitionLevel:revision:error:', {'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNRecognizeTextRequest', b'usesLanguageCorrection', {'retval': {'type': 'Z'}})
    r(b'VNRecognizedPointsObservation', b'identifiedPointsForGroupKey_error_', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNRecognizedPointsObservation', b'keypointsMultiArrayAndReturnError:', {'arguments': {2: {'type_modifier': b'o'}}})
    r(b'VNRecognizedPointsObservation', b'recognizedPointForKey:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNRecognizedPointsObservation', b'recognizedPointsForGroupKey:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNRecognizedText', b'boundingBoxForRange:error:', {'arguments': {3: {'type_modifier': b'o'}}})
    r(b'VNRequest', b'completionHandler', {'retval': {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}})
    r(b'VNRequest', b'initWithCompletionHandler:', {'arguments': {2: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNRequest', b'preferBackgroundProcessing', {'retval': {'type': 'Z'}})
    r(b'VNRequest', b'setPreferBackgroundProcessing:', {'arguments': {2: {'type': 'Z'}}})
    r(b'VNRequest', b'setUsesCPUOnly:', {'arguments': {2: {'type': 'Z'}}})
    r(b'VNRequest', b'usesCPUOnly', {'retval': {'type': 'Z'}})
    r(b'VNSequenceRequestHandler', b'performRequests:onCGImage:error:', {'retval': {'type': 'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onCGImage:orientation:error:', {'retval': {'type': 'Z'}, 'arguments': {5: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onCIImage:error:', {'retval': {'type': 'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onCIImage:orientation:error:', {'retval': {'type': 'Z'}, 'arguments': {5: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onCMSampleBuffer:error:', {'retval': {'type': b'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onCMSampleBuffer:orientation:error:', {'retval': {'type': b'Z'}, 'arguments': {5: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onCVPixelBuffer:error:', {'retval': {'type': 'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onCVPixelBuffer:orientation:error:', {'retval': {'type': 'Z'}, 'arguments': {5: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onImageData:error:', {'retval': {'type': 'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onImageData:orientation:error:', {'retval': {'type': 'Z'}, 'arguments': {5: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onImageURL:error:', {'retval': {'type': 'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNSequenceRequestHandler', b'performRequests:onImageURL:orientation:error:', {'retval': {'type': 'Z'}, 'arguments': {5: {'type_modifier': b'o'}}})
    r(b'VNStatefulRequest', b'frameAnalysisSpacing', {'retval': {'type': b'{_CMTime=qiIq}'}})
    r(b'VNStatefulRequest', b'initWithCompletionHandler:', {'arguments': {2: {'callable': {'retval': {'type': b'@?'}, 'arguments': {0: {'type': b'^v'}}}}}})
    r(b'VNStatefulRequest', b'initWithFrameAnalysisSpacing:completionHandler:', {'arguments': {2: {'type': b'{_CMTime=qiIq}'}, 3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNStatefulRequest', b'requestFrameAnalysisSpacing', {'retval': {'type': b'{_CMTime=qiIq}'}})
    r(b'VNTargetedImageRequest', b'initWithCompletionHandler:', {'arguments': {2: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCGImage:completionHandler:', {'arguments': {3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCGImage:options:completionHandler:', {'arguments': {4: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCGImage:orientation:options:completionHandler:', {'arguments': {5: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCIImage:completionHandler:', {'arguments': {3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCIImage:options:completionHandler:', {'arguments': {4: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCIImage:orientation:options:completionHandler:', {'arguments': {5: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCMSampleBuffer:options:completionHandler:', {'arguments': {4: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCMSampleBuffer:orientation:options:completionHandler:', {'arguments': {5: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCVPixelBuffer:completionHandler:', {'arguments': {3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCVPixelBuffer:options:completionHandler:', {'arguments': {4: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedCVPixelBuffer:orientation:options:completionHandler:', {'arguments': {5: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedImageData:completionHandler:', {'arguments': {3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedImageData:options:completionHandler:', {'arguments': {4: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedImageData:orientation:options:completionHandler:', {'arguments': {5: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedImageURL:completionHandler:', {'arguments': {3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedImageURL:options:completionHandler:', {'arguments': {4: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTargetedImageRequest', b'initWithTargetedImageURL:orientation:options:completionHandler:', {'arguments': {5: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTrackObjectRequest', b'initWithCompletionHandler:', {'arguments': {2: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTrackObjectRequest', b'initWithDetectedObjectObservation:completionHandler:', {'arguments': {3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTrackRectangleRequest', b'initWithCompletionHandler:', {'arguments': {2: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTrackRectangleRequest', b'initWithRectangleObservation:completionHandler:', {'arguments': {3: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTrackingRequest', b'initWithCompletionHandler:', {'arguments': {2: {'callable': {'retval': {'type': b'v'}, 'arguments': {0: {'type': b'^v'}, 1: {'type': b'@'}, 2: {'type': b'@'}}}}}})
    r(b'VNTrackingRequest', b'isLastFrame', {'retval': {'type': 'Z'}})
    r(b'VNTrackingRequest', b'setLastFrame:', {'arguments': {2: {'type': 'Z'}}})
    r(b'VNVideoProcessor', b'addRequest:processingOptions:error:', {'retval': {'type': b'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNVideoProcessor', b'addRequest:withProcessingOptions:error:', {'retval': {'type': b'Z'}, 'arguments': {4: {'type_modifier': b'o'}}})
    r(b'VNVideoProcessor', b'analyzeTimeRange:error:', {'retval': {'type': b'Z'}, 'arguments': {2: {'type': b'{_CMTimeRange={_CMTime=qiIq}{_CMTime=qiIq}}'}, 3: {'type_modifier': b'o'}}})
    r(b'VNVideoProcessor', b'analyzeWithTimeRange:error:', {'retval': {'type': b'Z'}, 'arguments': {2: {'type': b'{_CMTimeRange={_CMTime=qiIq}{_CMTime=qiIq}}'}, 3: {'type_modifier': b'o'}}})
    r(b'VNVideoProcessor', b'removeRequest:error:', {'retval': {'type': b'Z'}, 'arguments': {3: {'type_modifier': b'o'}}})
    r(b'null', b'hasMinimumPrecision:forRecall:', {'retval': {'type': b'Z'}})
    r(b'null', b'hasMinimumRecall:forPrecision:', {'retval': {'type': b'Z'}})
    r(b'null', b'hasPrecisionRecallCurve', {'retval': {'type': b'Z'}})
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNDetectHumanBodyPoseRequest(TestCase):
    @min_os_level("10.16")
    def testConstants10_16(self):

        self.assertIsInstance(Vision.VNBodyLandmarkKeyNose, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyLeftEye, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRightEye, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyLeftEar, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRightEar, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyLeftShoulder, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRightShoulder, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyNeck, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyLeftElbow, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRightElbow, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyLeftWrist, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRightWrist, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyLeftHip, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRightHip, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRoot, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyLeftKnee, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRightKnee, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyLeftAnkle, str)
        self.assertIsInstance(Vision.VNBodyLandmarkKeyRightAnkle, str)
        self.assertIsInstance(Vision.VNBodyLandmarkRegionKeyFace, str)
        self.assertIsInstance(Vision.VNBodyLandmarkRegionKeyTorso, str)
        self.assertIsInstance(Vision.VNBodyLandmarkRegionKeyLeftArm, str)
        self.assertIsInstance(Vision.VNBodyLandmarkRegionKeyRightArm, str)
        self.assertIsInstance(Vision.VNBodyLandmarkRegionKeyLeftLeg, str)
        self.assertIsInstance(Vision.VNBodyLandmarkRegionKeyRightLeg, str)

        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameNose, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameLeftEye, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameRightEye, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameLeftEar, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameRightEar, str)
        self.assertIsInstance(
            Vision.VNHumanBodyPoseObservationJointNameLeftShoulder, str
        )
        self.assertIsInstance(
            Vision.VNHumanBodyPoseObservationJointNameRightShoulder, str
        )
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameNeck, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameLeftElbow, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameRightElbow, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameLeftWrist, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameRightWrist, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameLeftHip, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameRightHip, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameRoot, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameLeftKnee, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameRightKnee, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameLeftAnkle, str)
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointNameRightAnkle, str)

        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointsGroupNameFace, str)
        self.assertIsInstance(
            Vision.VNHumanBodyPoseObservationJointsGroupNameTorso, str
        )
        self.assertIsInstance(
            Vision.VNHumanBodyPoseObservationJointsGroupNameLeftArm, str
        )
        self.assertIsInstance(
            Vision.VNHumanBodyPoseObservationJointsGroupNameRightArm, str
        )
        self.assertIsInstance(
            Vision.VNHumanBodyPoseObservationJointsGroupNameLeftLeg, str
        )
        self.assertIsInstance(
            Vision.VNHumanBodyPoseObservationJointsGroupNameRightLeg, str
        )
        self.assertIsInstance(Vision.VNHumanBodyPoseObservationJointsGroupNameAll, str)

        self.assertEqual(Vision.VNDetectHumanBodyPoseRequestRevision1, 1)

    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertArgIsOut(
            Vision.VNDetectHumanBodyPoseRequest.supportedRecognizedPointKeysForRevision_error_,
            1,
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanBodyPoseRequest.supportedRecognizedPointGroupKeysForRevision_error_,
            1,
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanBodyPoseRequest.supportedIdentifiedPointKeysForRevision_error_,
            1,
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanBodyPoseRequest.supportedIdentifiedPointGroupKeysForRevision_error_,
            1,
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanBodyPoseRequest.supportedJointNamesForRevision_error_, 1
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanBodyPoseRequest.supportedJointsGroupNamesForRevision_error_,
            1,
        )
        self.assertArgIsOut(
            Vision.VNHumanBodyPoseObservation.recognizedPointForJointName_error_, 1
        )
        self.assertArgIsOut(
            Vision.VNHumanBodyPoseObservation.recognizedPointsForJointsGroupName_error_,
            1,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNDetectFaceLandmarksRequest(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNRequestFaceLandmarksConstellationNotDefined, 0)
        self.assertEqual(Vision.VNRequestFaceLandmarksConstellation65Points, 1)
        self.assertEqual(Vision.VNRequestFaceLandmarksConstellation76Points, 2)

        self.assertEqual(Vision.VNDetectFaceLandmarksRequestRevision2, 2)
        self.assertEqual(Vision.VNDetectFaceLandmarksRequestRevision3, 3)

    @min_os_level("10.15")
    def test_methods10_15(self):
        self.assertResultIsBOOL(
            Vision.VNDetectFaceLandmarksRequest.revision_supportsConstellation_
        )



from PyObjCTools.TestSupport import TestCase
import Vision


class TestVNDetectFaceLandmarks(TestCase):
    def test_constants(self):
        self.assertEqual(Vision.VNDetectFaceLandmarksRequestRevision1, 1)
        self.assertEqual(Vision.VNDetectFaceLandmarksRequestRevision2, 2)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNDetectHumanHandPoseRequest(TestCase):
    @min_os_level("10.16")
    def test_constants10_16(self):
        self.assertIsInstance(Vision.VNHandLandmarkKeyThumbCMC, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyThumbMP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyThumbIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyThumbTIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyIndexMCP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyIndexPIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyIndexDIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyIndexTIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyMiddleMCP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyMiddlePIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyMiddleDIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyMiddleTIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyRingMCP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyRingPIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyRingDIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyRingTIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyLittleMCP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyLittlePIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyLittleDIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkKeyLittleTIP, str)
        self.assertIsInstance(Vision.VNHandLandmarkRegionKeyThumb, str)
        self.assertIsInstance(Vision.VNHandLandmarkRegionKeyMiddleFinger, str)
        self.assertIsInstance(Vision.VNHandLandmarkRegionKeyIndexFinger, str)
        self.assertIsInstance(Vision.VNHandLandmarkRegionKeyRingFinger, str)
        self.assertIsInstance(Vision.VNHandLandmarkRegionKeyLittleFinger, str)

        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameWrist, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameThumbCMC, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameThumbMP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameThumbIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameThumbTip, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameIndexMCP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameIndexPIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameIndexDIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameIndexTip, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameMiddleMCP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameMiddlePIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameMiddleDIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameMiddleTip, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameRingMCP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameRingPIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameRingDIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameRingTip, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameLittleMCP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameLittlePIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameLittleDIP, str)
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointNameLittleTip, str)

        self.assertIsInstance(
            Vision.VNHumanHandPoseObservationJointsGroupNameThumb, str
        )
        self.assertIsInstance(
            Vision.VNHumanHandPoseObservationJointsGroupNameIndexFinger, str
        )
        self.assertIsInstance(
            Vision.VNHumanHandPoseObservationJointsGroupNameMiddleFinger, str
        )
        self.assertIsInstance(
            Vision.VNHumanHandPoseObservationJointsGroupNameRingFinger, str
        )
        self.assertIsInstance(
            Vision.VNHumanHandPoseObservationJointsGroupNameLittleFinger, str
        )
        self.assertIsInstance(Vision.VNHumanHandPoseObservationJointsGroupNameAll, str)

        self.assertEqual(Vision.VNDetectHumanHandPoseRequestRevision1, 1)

    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertArgIsOut(
            Vision.VNDetectHumanHandPoseRequest.supportedRecognizedPointKeysForRevision_error_,
            1,
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanHandPoseRequest.supportedRecognizedPointGroupKeysForRevision_error_,
            1,
        )

        self.assertArgIsOut(
            Vision.VNDetectHumanHandPoseRequest.supportedIdentifiedPointKeysForRevision_error_,
            1,
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanHandPoseRequest.supportedIdentifiedPointGroupKeysForRevision_error_,
            1,
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanHandPoseRequest.supportedJointNamesForRevision_error_, 1
        )
        self.assertArgIsOut(
            Vision.VNDetectHumanHandPoseRequest.supportedJointsGroupNamesForRevision_error_,
            1,
        )

        self.assertArgIsOut(
            Vision.VNHumanHandPoseObservation.recognizedPointForJointName_error_, 1
        )
        self.assertArgIsOut(
            Vision.VNHumanHandPoseObservation.recognizedPointsForJointsGroupName_error_,
            1,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNClassifyImageRequest(TestCase):
    @min_os_level("10.15")
    def test_methods(self):
        self.assertArgIsOut(
            Vision.VNClassifyImageRequest.knownClassificationsForRevision_error_, 1
        )

    def test_constants(self):
        self.assertEqual(Vision.VNClassifyImageRequestRevision1, 1)



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNGeometryUtils(TestCase):
    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertArgIsOut(Vision.VNGeometryUtils.boundingCircleForContour_error_, 1)
        self.assertArgIsOut(Vision.VNGeometryUtils.boundingCircleForPoints_error_, 1)

        # XXX: SIMD argument
        # self.assertArgIsIn(
        #     Vision.VNGeometryUtils.boundingCircleForSIMDPoints_pointCount_error_, 0
        # )
        # self.assertArgSizeInArg(
        #     Vision.VNGeometryUtils.boundingCircleForSIMDPoints_pointCount_error_, 0, 1
        # )
        # self.assertArgIsOut(
        #     Vision.VNGeometryUtils.boundingCircleForSIMDPoints_pointCount_error_, 2
        # )

        self.assertResultIsBOOL(
            Vision.VNGeometryUtils.calculateArea_forContour_orientedArea_error_
        )
        self.assertArgIsIn(
            Vision.VNGeometryUtils.calculateArea_forContour_orientedArea_error_, 0
        )
        self.assertArgIsBOOL(
            Vision.VNGeometryUtils.calculateArea_forContour_orientedArea_error_, 2
        )
        self.assertArgIsOut(
            Vision.VNGeometryUtils.calculateArea_forContour_orientedArea_error_, 3
        )

        self.assertResultIsBOOL(
            Vision.VNGeometryUtils.calculatePerimeter_forContour_error_
        )
        self.assertArgIsIn(
            Vision.VNGeometryUtils.calculatePerimeter_forContour_error_, 0
        )
        self.assertArgIsOut(
            Vision.VNGeometryUtils.calculatePerimeter_forContour_error_, 2
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import Vision


class TestVNGeometry(TestCase):
    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertResultIsBOOL(Vision.VNCircle.containsPoint_)
        self.assertResultIsBOOL(
            Vision.VNCircle.containsPoint_inCircumferentialRingOfWidth_
        )

        # XXX: VNContour.normalizedPoints
